@@ -1,8 +1,5 @@
 # -*- coding: utf-8 -*-
 """
-Authors: Tim Hessels
-Module: Collect/SRTM
-
 
 Description:
 This module downloads DEM data from http://earlywarning.usgs.gov/hydrodata/.
@@ -12,7 +9,7 @@ Examples:
 from pyWAPOR.Collect import SRTM
 SRTM.DEM(Dir='C:/TempDEM4/', latlim=[29, 32], lonlim=[-113, -109])
 """
-from .DEM import main as DEM
+from pywapor.collect.SRTM.DEM import main as DEM
 
 __all__ = ['DEM']
 
